import json
import typing


class LambdaEvent:
    class RequestContext:
        def __init__(self, data):
            self.resourceId = ['resourceId']
            self.resourcePath = ['resourcePath']
            self.httpMethod = ['httpMethod']
            self.requestTime = ['requestTime']
            self.path = ['path']
            self.domainName = ['domainName']

    def __init__(self, data):
        self.headers = data['headers'] or dict()
        self.httpMethod = data['httpMethod']
        self.path = data['path']
        self.queryStringParameters = data['queryStringParameters'] or dict()
        self._body = data['body']
        self.requestContext = self.RequestContext(data['requestContext'])

    @property
    def body(self):
        if self._body:
            return json.loads(self._body)
        return self._body

    @property
    def lower_headers(self) -> typing.Dict:
        return {k.lower(): v for k, v in self.headers.items()}



import json
import os

import boto3
from botocore.exceptions import ClientError

from . import exceptions


class SecretsManager:
    __secrets = None

    @classmethod
    def __get_secrets(cls, secret_name: str):
        try:
            sm_client = boto3.client('secretsmanager')
            r_secrets = sm_client.get_secret_value(
                SecretId=secret_name
            )
            return json.loads(r_secrets['SecretString'])
        except ClientError as e:
            # For a list of exceptions thrown, see
            # https://docs.aws.amazon.com/secretsmanager/latest/apireference/API_GetSecretValue.html
            raise exceptions.SecretsManagerException(e)

    @classmethod
    def get_secrets(cls):
        if not cls.__secrets:
            cls.__secrets = cls.__get_secrets(secret_name=cls.get_aws_secret_name())
        return cls.__secrets

    @classmethod
    def get_aws_secret_name(cls) -> str:
        return os.environ['AWS_SECRET_NAME']



from boto3.dynamodb import types

from . import encoders


class BaseModel:
    def __init__(self, data):
        self.__data = self.to_normalized_data(data)
        self.PK = self.data['PK']
        self.SK = self.data['SK']
        self.Type = self.data['Type']
        self.CreatedAt = self.data['CreatedAt']
        self.CreatedBy = self.data.get('CreatedBy')
        self.ModifiedAt = self.data.get('ModifiedAt')
        self.ModifiedBy = self.data.get('ModifiedBy')
        self.ExpiresAt = self.data.get('ExpiresAt')

    @property
    def data(self):
        return self.__data

    @staticmethod
    def to_normalized_data(data: dict) -> dict:
        """
        Converts low level dynamo json to normalized json
        """
        return {k: types.TypeDeserializer().deserialize(v) for k, v in data.items()}

    @property
    def to_json(self):
        return encoders.DBEncoder().serialize_to_json(self)



import logging.config
import os

logger = logging.getLogger(__name__)

LOG_LEVEL = getattr(logging, os.environ.get('LOG_LEVEL', 'DEBUG').upper())

LOGGING_CONFIG = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'simple': {
            'format': '[{levelname}] {message}',
            'style': '{',
        },
        'verbose': {
            'format': '[{asctime} {levelname}/{threadName}] {message}',
            'style': '{',
        },
        'verbose_plus': {
            'format': '[{asctime} {levelname}/{threadName} {pathname}] {message}',
            'style': '{',
        },
    },
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'verbose',
        },
    },
    'loggers': {
        '': {
            'handlers': ['console'],
            'level': LOG_LEVEL,
            'propagate': True,
        }
    },
}

logging.config.dictConfig(LOGGING_CONFIG)



import logging
from functools import wraps
from typing import Any, Dict, List

from botocore.exceptions import ClientError

from . import dynamodb, utils

logger = logging.getLogger(__name__)


def boto3_handler(raise_as, client_error_map: dict):
    def deco_func(func):
        @wraps(func)
        def wrapper_func(*args, **kwargs):
            try:
                response = func(*args, **kwargs)
                logger.debug(f'{__name__}, response: {response}')
                return response
            except ClientError as e:
                error_code = e.response['Error']['Code']
                logger.error(f'boto3 client error: {str(e)}, response: {e.response}, error code: {error_code}')
                if exc := client_error_map.get(error_code):
                    raise exc(e)
                raise raise_as()
            except Exception:  # Raise all other exceptions as is
                raise
        return wrapper_func  # true decorator
    return deco_func


def dynamodb_handler(client_err_map: Dict[str, Any], cancellation_err_maps: List[Dict[str, Any]]):
    def deco_func(func):
        @wraps(func)
        def wrapper_func(*args, **kwargs):
            try:
                response = func(*args, **kwargs)
                logger.debug(f'{func.__name__}, response: {response}')
                return response
            except ClientError as e:
                logger.error(f'ClientError detected: {e}')
                e_response = dynamodb.ErrorResponse(e.response)
                if e_response.CancellationReasons:
                    e_response.raise_for_cancellation_reasons(error_maps=cancellation_err_maps)
                if exc := client_err_map.get(e_response.Error.Code):
                    raise exc(e)
                raise
        return wrapper_func  # true decorator
    return deco_func


def lambda_response_handler(raise_as):
    """
    Passes through any exceptions that inherit from the
    :param raise_as:
    :return:
    """
    def deco_func(func):
        @wraps(func)
        def wrapper_func(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except raise_as as e:
                logger.exception(str(e))
                exc = e
            except Exception as e:
                logger.exception(str(e))
                exc = raise_as(e)
            return utils.build_lambda_response(
                status_code=400,
                exc=exc
            )
        return wrapper_func  # true decorator
    return deco_func
